import uuid

import transaction
from pyramid.view import view_config

from barker.models import Location, MachineLocation


@view_config(request_method='PUT', route_name='machine_location', renderer='json', permission='Everyone', trans=True)
def save(request):
    json = request.json_body
    item = MachineLocation(json['Machine'])
    item.location = request.dbsession.query(Location).filter(Location.name == json['Location']).first()
    request.dbsession.add(item)
    transaction.commit()
    item = request.dbsession.query(MachineLocation).filter(MachineLocation.id == item.id).first()
    return machine_location_info(item)


@view_config(request_method='POST', route_name='machine_location_id', renderer='json', permission='Machines',
             trans=True)
def update(request):
    id = uuid.UUID(request.matchdict['id'])
    json = request.json_body
    item = request.dbsession.query(MachineLocation).filter(MachineLocation.id == id).first()
    item.machine_name = json['Machine']
    item.location = request.dbsession.query(Location).filter(Location.name == json['Location']).first()
    transaction.commit()
    item = request.dbsession.query(MachineLocation).filter(MachineLocation.id == item.id).first()
    return machine_location_info(item)


@view_config(request_method='DELETE', route_name='machine_location_id', renderer='json', permission='Machines',
             trans=True)
def delete(request):
    id = uuid.UUID(request.matchdict['id'])
    item = request.dbsession.query(MachineLocation).filter(MachineLocation.id == id).first()
    request.dbsession.delete(item)
    transaction.commit()
    return {}


@view_config(request_method='GET', route_name='machine_location_id', renderer='json', permission='Authenticated')
def show_id(request):
    id = uuid.UUID(request.matchdict['id'])
    item = request.dbsession.query(MachineLocation).filter(MachineLocation.id == id).first()
    return machine_location_info(item)


@view_config(request_method='GET', route_name='machine_location', renderer='json', request_param='n',
             permission='Everyone')
def show_name(request):
    name = request.GET['n']
    item = request.dbsession.query(MachineLocation).filter(MachineLocation.machine_name == name).first()
    return machine_location_info(item)


@view_config(request_method='GET', route_name='machine_location_list', renderer='json', permission='Authenticated')
def show_list(request):
    list = request.dbsession.query(MachineLocation).order_by(MachineLocation.machine_name).all()
    machine_locations = []
    for item in list:
        machine_locations.append({
            'MachineLocationID': item.id,
            'Machine': item.machine_name,
            'Location': item.location.name
        })
    return machine_locations


def machine_location_info(item):
    if item is None:
        return {}
    return {
        'MachineLocationID': item.id,
        'Machine': item.machine_name,
        'Location': item.location.name
    }
