import uuid

import transaction

from pyramid.view import view_config
from sqlalchemy import or_

from barker.models import Modifier, ProductGroupModifier


@view_config(request_method='PUT', route_name='modifier', renderer='json', permission='Modifiers', trans=True)
def save(request):
    json = request.json_body
    item = Modifier(json['Name'], json['ShowInBill'], json['Price'])
    request.dbsession.add(item)
    add_groups(item, json['ProductGroupModifiers'], request.dbsession)
    transaction.commit()
    item = request.dbsession.query(Modifier).filter(Modifier.id == item.id).first()
    return modifier_info(item)


@view_config(request_method='POST', route_name='modifier_id', renderer='json', permission='Modifiers', trans=True)
def update(request):
    json = request.json_body
    item = request.dbsession.query(Modifier).filter(Modifier.id == uuid.UUID(request.matchdict['id'])).first()
    item.name = json['Name']
    item.show_in_bill = json['ShowInBill']
    item.price = json['Price']
    add_groups(item, json['ProductGroupModifiers'], request.dbsession)
    transaction.commit()
    item = request.dbsession.query(Modifier).filter(Modifier.id == item.id).first()
    return modifier_info(item)


@view_config(request_method='DELETE', route_name='modifier_id', renderer='json', permission='Modifiers', trans=True)
def delete(request):
    item = request.dbsession.query(Modifier).filter(Modifier.id == uuid.UUID(request.matchdict['id'])).first()
    request.dbsession.delete(item)
    transaction.commit()
    return {}


@view_config(request_method='GET', route_name='modifier_id', renderer='json', permission='Authenticated')
def show_id(request):
    id = uuid.UUID(request.matchdict['id'])
    modifier = request.dbsession.query(Modifier).filter(Modifier.id == id).first()
    return modifier_info(modifier)


@view_config(request_method='GET', route_name='modifier_list', renderer='json', permission='Authenticated')
def show_list(request):
    list = request.dbsession.query(Modifier).order_by(Modifier.name).all()
    modifiers = []
    for item in list:
        modifiers.append({
            'ModifierID': item.id,
            'Name': item.name,
            'ShowInBill': item.show_in_bill,
            'Price': item.price
        })
    return modifiers


@view_config(request_method='GET', route_name='modifier_list', renderer='json', request_param='pg',
             permission='Authenticated')
def show_for_pg(request):
    product_group_id = uuid.UUID(request.GET['pg'])
    pgm_list = request.dbsession.query(
        ProductGroupModifier
    ).filter(
        or_(ProductGroupModifier.product_group == None, ProductGroupModifier.product_group_id == product_group_id)
    ).all()

    pg_modifiers = []
    for item in pgm_list:
        pg_modifiers.append({
            'ModifierID': item.modifier_id,
            'Name': item.modifier.name,
            'ShowInBill': item.modifier.show_in_bill,
            'Price': item.modifier.price
        })
    return pg_modifiers


def modifier_info(modifier):
    return {
        'ModifierID': modifier.id,
        'Name': modifier.name,
        'ShowInBill': modifier.show_in_bill,
        'Price': modifier.price,
        'ProductGroupModifiers': [
            {
                'ProductGroupModifierID': i.id,
                'ProductGroup': {'ProductGroupID': i.product_group_id} if i.product_group_id is not None else None,
                'ShowAutomatically': i.show_automatically
            } for i in modifier.product_groups]
    }


def add_groups(modifier, groups, dbsession):
    all = [p for p in groups if p['ProductGroup'] is None]
    if len(all) > 0:
        old = [p for p in modifier.product_groups if p.product_group_id is None]
        if len(old) != 1:
            for item in modifier.product_groups:
                dbsession.delete(item)
            modifier.product_groups.clear()
            modifier.product_groups.append(ProductGroupModifier(None, None, False))
    else:
        new = {uuid.UUID(p['ProductGroup']['ProductGroupID']) for p in groups}
        old = {p.product_group_id for p in modifier.product_groups}
        for pg_id in new - old:
            modifier.product_groups.append(ProductGroupModifier(pg_id, None, False))

        gp = [p for p in modifier.product_groups if p.product_group_id in (old - new)]
        for item in gp:
            dbsession.delete(item)
