import uuid

import transaction
from pyramid.response import Response
from pyramid.view import view_config

from barker.models import Product


@view_config(request_method='PUT', route_name='product', renderer='json', permission='Products', trans=True)
def save(request):
    json = request.json_body
    item = Product(json['Name'], json['Units'], json['ProductGroupID'], json['VatID'], json['ServiceTaxID'],
                   json['Price'], json['HasHappyHour'], json['IsNotAvailable'], json['Quantity'], json['IsActive'],
                   json['SortOrder'])
    request.dbsession.add(item)
    transaction.commit()
    item = request.dbsession.query(Product).filter(Product.id == item.id).first()
    return product_info(item)


@view_config(request_method='POST', route_name='product_id', renderer='json', permission='Products', trans=True)
def update(request):
    item = request.dbsession.query(Product).filter(Product.id == uuid.UUID(request.matchdict['id'])).first()
    json = request.json_body
    item.name = json['Name']
    item.units = json['Units']
    item.product_group_id = json['ProductGroupID']
    item.vat_id = json['VatID']
    item.service_tax_id = json['ServiceTaxID']
    item.price = json['Price']
    item.has_happy_hour = json['HasHappyHour']
    item.is_not_available = json['IsNotAvailable']
    item.quantity = json['Quantity']
    item.is_active = json['IsActive']
    item.sort_order = json['SortOrder']
    transaction.commit()
    item = request.dbsession.query(Product).filter(Product.id == item.id).first()
    return product_info(item)


@view_config(request_method='DELETE', route_name='product_id', renderer='json', permission='Products', trans=True)
def delete(request):
    item = request.dbsession.query(Product).filter(Product.id == uuid.UUID(request.matchdict['id'])).first()
    if item is None:
        response = Response("Product not Found")
        response.status_int = 500
        return response
    else:
        response = Response("Product deletion not implemented")
        response.status_int = 500
        return response


@view_config(request_method='GET', route_name='product_id', renderer='json', permission='Authenticated')
def show_id(request):
    id = uuid.UUID(request.matchdict['id'])
    item = request.dbsession.query(Product).filter(Product.id == id).first()
    return product_info(item)


@view_config(request_method='GET', route_name='product_list', renderer='json', permission='Authenticated')
def show_list(request):
    active = request.GET.get('a', None)
    list = request.dbsession.query(Product)
    if active is not None:
        list = list.filter(Product.is_active == active)
    list = list.order_by(Product.product_group_id, Product.sort_order).order_by(Product.name).all()
    products = []
    for item in list:
        products.append(product_info(item))
    return products


@view_config(request_method='POST', route_name='product_list', renderer='json', permission='Products', trans=True)
def sort_order(request):
    json = request.json_body
    index, last_group = 0, None
    for item in json:
        product_id = uuid.UUID(item['ProductID'])
        product_group_id = uuid.UUID(item['ProductGroupID'])
        if last_group != product_group_id:
            index = 0
            last_group = product_group_id
        request.dbsession.query(
            Product
        ).filter(
            Product.id == product_id
        ).update(
            {Product.sort_order: index}
        )
    return True


def product_info(item):
    return {
        'ProductID': item.id,
        'Name': item.name,
        'Units': item.units,
        'ProductGroup': {'ProductGroupID': item.product_group_id, 'Name': item.product_group.name},
        'Vat': {'TaxID': item.vat_id, 'Name': item.vat.name, 'Rate': item.vat.rate},
        'ServiceTax': {'TaxID': item.service_tax_id, 'Name': item.service_tax.name, 'Rate': item.service_tax.rate},
        'Price': item.price,
        'HasHappyHour': item.has_happy_hour,
        'IsNotAvailable': item.is_not_available,

        'IsActive': item.is_active,
        'SortOrder': item.sort_order
    }
