import uuid

import transaction
from pyramid.response import Response
from pyramid.view import view_config

from barker.exceptions import ValidationFailure
from barker.models import Product, ProductGroup


@view_config(request_method='PUT', route_name='product_group', renderer='json', permission='Products', trans=True)
def save(request):
    json = request.json_body
    item = ProductGroup(json['Name'], json['DiscountLimit'], json['IsModifierCompulsory'], json['GroupType'],
                        json['IsActive'], json['SortOrder'])
    request.dbsession.add(item)
    transaction.commit()
    item = request.dbsession.query(ProductGroup).filter(ProductGroup.id == item.id).first()
    return product_group_info(item)


@view_config(request_method='POST', route_name='product_group_id', renderer='json', permission='Products', trans=True)
def update(request):
    item = request.dbsession.query(ProductGroup).filter(ProductGroup.id == uuid.UUID(request.matchdict['id'])).first()
    json = request.json_body
    item.name = json['Name']
    item.discount_limit = json['DiscountLimit']
    item.is_modifier_compulsory = json['IsModifierCompulsory']
    item.group_type = json['GroupType']
    item.is_active = json['IsActive']
    item.sort_order = json['SortOrder']
    transaction.commit()
    item = request.dbsession.query(ProductGroup).filter(ProductGroup.id == item.id).first()
    return product_group_info(item)


@view_config(request_method='DELETE', route_name='product_group_id', renderer='json', permission='Products', trans=True)
def delete(request):
    item = request.dbsession.query(ProductGroup).filter(ProductGroup.id == uuid.UUID(request.matchdict['id'])).first()
    if item is None:
        response = Response("Product Group not Found")
        response.status_int = 500
        return response
    elif item.is_fixture:
        transaction.abort()
        raise ValidationFailure("{0} is a fixture and cannot be edited or deleted.".format(item.name))
    else:
        response = Response("Product Group deletion not implemented")
        response.status_int = 500
        return response


@view_config(request_method='GET', route_name='product_group_id', renderer='json', permission='Authenticated')
def show_id(request):
    id = uuid.UUID(request.matchdict['id'])
    item = request.dbsession.query(ProductGroup).filter(ProductGroup.id == id).first()
    return product_group_info(item)


@view_config(request_method='GET', route_name='product_group_list', renderer='json', permission='Authenticated')
def show_list(request):
    list = request.dbsession.query(ProductGroup).order_by(ProductGroup.sort_order).order_by(ProductGroup.name).all()
    product_groups = []
    for item in list:
        product_groups.append(product_group_info(item))
    return product_groups


@view_config(request_method='GET', route_name='product_group_list', renderer='json', request_param='s',
             permission='Authenticated')
def sale_list(request):
    list = request.dbsession.query(
        ProductGroup
    ).filter(ProductGroup.is_active == True).order_by(ProductGroup.sort_order).order_by(ProductGroup.name).all()
    product_groups = []
    for item in list:
        products = request.dbsession.query(
            Product
        ).filter(
            Product.product_group_id == item.id
        ).filter(Product.is_active == True).order_by(Product.sort_order).order_by(Product.name).all()
        if len(products) > 0:
            pg = product_group_info(item)
            pg['Products'] = []
            for p in products:
                pg['Products'].append({
                    'ProductID': p.id,
                    'Name': p.name,
                    'Units': p.units,
                    'Vat': {'TaxID': p.vat.id, 'Name': p.vat.name, 'Rate': p.vat.rate},
                    'ServiceTax': {'TaxID': p.service_tax.id, 'Name': p.service_tax.name, 'Rate': p.service_tax.rate},
                    'Price': p.price,
                    'HasHappyHour': p.has_happy_hour,
                    'IsActive': p.is_active,
                    'IsNotAvailable': p.is_not_available,
                    'SortOrder': p.sort_order,
                    'Quantity': p.quantity})
            product_groups.append(pg)
    return product_groups


@view_config(request_method='POST', route_name='product_group_list', renderer='json', permission='Products', trans=True)
def sort_order(request):
    json = request.json_body
    for index, item in enumerate(json):
        request.dbsession.query(
            ProductGroup
        ).filter(
            ProductGroup.id == uuid.UUID(item['ProductGroupID'])
        ).update(
            {ProductGroup.sort_order: index}
        )
    return True


@view_config(request_method='GET', route_name='product_group_type_list', renderer='json', permission='Authenticated')
def product_group_type_list(request):
    list = request.dbsession.query(
        ProductGroup.group_type
    ).group_by(
        ProductGroup.group_type
    ).order_by(
        ProductGroup.group_type
    ).all()
    return [item.group_type for item in list]


def product_group_info(item):
    return {
        'ProductGroupID': item.id,
        'Name': item.name,
        'DiscountLimit': item.discount_limit,
        'IsModifierCompulsory': item.is_modifier_compulsory,
        'GroupType': item.group_type,

        'IsActive': item.is_active,
        'SortOrder': item.sort_order
    }
