import datetime
import uuid

from barker.models import Voucher, User, Settlement
from barker.models.master import SettleOption
from pyramid.view import view_config
from sqlalchemy.orm import joinedload

__author__ = 'tanshu'


@view_config(request_method='GET', route_name='active_cashiers', renderer='json', permission='Cashier Checkout',
             request_param=('s', 'f'))
def active_cashiers(request):
    start_date = datetime.datetime.strptime(request.GET['s'], '%d-%b-%Y %H:%M')
    finish_date = datetime.datetime.strptime(request.GET['f'], '%d-%b-%Y %H:%M')

    user_ids = request.dbsession.query(Voucher.user_id).filter(
        Voucher.date >= start_date,
        Voucher.date <= finish_date
    ).distinct()

    users = request.dbsession.query(
        User
    ).filter(
        User.id.in_(user_ids)
    ).order_by(
        User.name
    ).all()

    return [{'UserID': u.id, 'Name': u.name} for u in users]


@view_config(request_method='GET', route_name='checkout', renderer='json', permission='Cashier Checkout',
             request_param=('s', 'f'))
def check_me_out(request):
    id = uuid.UUID(request.matchdict['id'])
    start_date = datetime.datetime.strptime(request.GET['s'], '%d-%b-%Y %H:%M')
    finish_date = datetime.datetime.strptime(request.GET['f'], '%d-%b-%Y %H:%M')

    vouchers = request.dbsession.query(
        Voucher
    ).options(
        joinedload(Voucher.settlements, innerjoin=True).joinedload(Settlement.settle_option, innerjoin=True),
        joinedload(Voucher.customer, innerjoin=True)
    ).filter(
        Voucher.date >= start_date,
        Voucher.date <= finish_date,
        Voucher.user_id == id
    ).order_by(
        Voucher.voucher_type
    ).order_by(
        Voucher.bill_id
    ).all()

    info = {}
    for item in vouchers:
        if item.is_void:
            if SettleOption.VOID() not in info:
                info[SettleOption.VOID()] = []
            amount = next(s.amount * -1 for s in item.settlements if s.settled == SettleOption.AMOUNT())
            info[SettleOption.VOID()].append({
                'Date': item.date.strftime('%d-%b-%Y %H:%M:%S'),
                'BillID': item.full_bill_id,
                'Customer': item.customer.name,
                'Amount': amount
            })
        else:
            for so in (so for so in item.settlements if so.settle_option.show_in_choices):
                if so.settled not in info:
                    info[so.settled] = []
                info[so.settled].append({
                    'Date': item.date.strftime('%d-%b-%Y %H:%M:%S'),
                    'BillID': item.full_bill_id,
                    'Customer': item.customer.name,
                    'Amount': so.amount
                })
    return info
