import uuid

import transaction
from pyramid.view import view_config

from barker.models import Tax


@view_config(request_method='PUT', route_name='tax', renderer='json', permission='Taxes', trans=True)
def save(request):
    json = request.json_body
    item = Tax(json['Name'], json['Rate'])
    request.dbsession.add(item)
    transaction.commit()
    item = request.dbsession.query(Tax).filter(Tax.id == item.id).first()
    return tax_info(item)


@view_config(request_method='POST', route_name='tax_id', renderer='json', permission='Taxes', trans=True)
def update(request):
    json = request.json_body
    item = request.dbsession.query(Tax).filter(Tax.id == uuid.UUID(request.matchdict['id'])).first()
    item.name = json['Name']
    item.rate = json['Rate']
    transaction.commit()
    item = request.dbsession.query(Tax).filter(Tax.id == item.id).first()
    return tax_info(item)


@view_config(request_method='DELETE', route_name='tax_id', renderer='json', permission='Taxes', trans=True)
def delete(request):
    item = request.dbsession.query(Tax).filter(Tax.id == uuid.UUID(request.matchdict['id'])).first()
    request.dbsession.delete(item)
    transaction.commit()
    return {}


@view_config(request_method='GET', route_name='tax_id', renderer='json', permission='Authenticated')
def show_id(request):
    id = uuid.UUID(request.matchdict['id'])
    item = request.dbsession.query(Tax).filter(Tax.id == id).first()
    return tax_info(item)


@view_config(request_method='GET', route_name='tax_list', renderer='json', permission='Authenticated')
def show_list(request):
    list = request.dbsession.query(Tax).order_by(Tax.name).all()
    taxes = []
    for item in list:
        taxes.append({
            'TaxID': item.id,
            'Name': item.name,
            'Rate': item.rate,
        })
    return taxes


def tax_info(item):
    return {
        'TaxID': item.id,
        'Name': item.name,
        'Rate': item.rate
    }
