import {DataSource} from '@angular/cdk/collections';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import {map, tap} from 'rxjs/operators';
import {Observable, of as observableOf, merge} from 'rxjs';
import {GuestBook} from "../guest-book";

/**
 * Data source for the GuestBookList view. This class should
 * encapsulate all logic for fetching and manipulating the displayed data
 * (including sorting, pagination, and filtering).
 */
export class GuestBookListDataSource extends DataSource<GuestBook> {
  private dataObservable: Observable<GuestBook[]>;

  constructor(
    private paginator: MatPaginator,
    private sort: MatSort,
    public data: GuestBook[]
  ) {
    super();
  }

  connect(): Observable<GuestBook[]> {
    this.dataObservable = observableOf(this.data);
    const dataMutations = [
      this.dataObservable,
      this.paginator.page,
      this.sort.sortChange
    ];

    return merge(...dataMutations).pipe(
      map((x: any) => {
        return this.getPagedData(this.getSortedData([...this.data]));
      }),
      tap((x: GuestBook[]) => this.paginator.length = x.length)
    );

  }

  /**
   *  Called when the table is being destroyed. Use this function, to clean up
   * any open connections or free any held resources that were set up during connect.
   */
  disconnect() {
  }

  private getPagedData(data: GuestBook[]) {
    const startIndex = this.paginator.pageIndex * this.paginator.pageSize;
    return data.splice(startIndex, this.paginator.pageSize);
  }

  private getSortedData(data: GuestBook[]) {
    if (!this.sort.active || this.sort.direction === '') {
      return data;
    }

    return data.sort((a, b) => {
      const isAsc = this.sort.direction === 'asc';
      switch (this.sort.active) {
        case 'name':
          return compare(a.name, b.name, isAsc);
        case 'id':
          return compare(+a.id, +b.id, isAsc);
        default:
          return 0;
      }
    });
  }
}

function compare(a, b, isAsc) {
  return (a < b ? -1 : 1) * (isAsc ? 1 : -1);
}
